public class FFBNet {
  int vecLen;
  int depth;
  float scale;
  float[] params;
  float[] flips;
  
  FFBNet(int vecLen,int depth) {
    this.vecLen = vecLen;
    this.depth = depth;
    scale = 1f / (float)Math.sqrt(vecLen);
    params = new float[2 * vecLen * depth];
    flips = new float[vecLen];
    for (int i = 0; i < vecLen; i++) {
      flips[i] = Math.sin(i * 1.895) < 0 ? -scale : scale;
    }
    for (int i = 0; i < params.length; i++) {
      params[i] = scale;
    }
  }

  void recall(float[] result, float[] input) {
    for (int i = 0; i < vecLen; i++) {
      result[i] = input[i] * flips[i];
    }
    int paramIdx = 0;
    for (int i = 0; i < depth; i++) {
      wht(result);
      for (int j = 0; j < vecLen; j++) {
        int signBit = result[j] < 0 ? 0 : 1;
        result[j] *= params[paramIdx + signBit];
        paramIdx += 2;
      }
    }
    wht(result);
  }
  
  void wht(float[] vec) {
  final int n = vecLen;
  int hs = 1;
  while (hs < n) {
    int i = 0;
    while (i < n) {
      final int j = i + hs;
      while (i < j) {
        float a = vec[i];
        float b = vec[i + hs];
        vec[i] = a + b;
        vec[i + hs] = a - b;
        i += 1;
      }
      i += hs;
    }
    hs += hs;
  }
}

}
